from pyResearchInsights.Scraper import scraper_main
from pyResearchInsights.Cleaner import cleaner_main
from pyResearchInsights.Analyzer import analyzer_main
from pyResearchInsights.NLP_Engine import nlp_engine_main
from pyResearchInsights.common_functions import pre_processing, arguments_parser, end_process
from pyResearchInsights.system_functions import tarballer, rm_original_folder